﻿// Author: Orlys
// Github: https://github.com/Orlys

namespace Yuuna
{
    using System;
    using Yuuna.Contracts.Modules;
    using Yuuna.Recognition.Speech;
    using System.Threading.Tasks;
    using System.Reflection;
    using Yuuna.Contracts.Interaction;
    using Yuuna.TextSegmention;
    using System.Diagnostics;
    using Microsoft.AspNetCore;
    using Microsoft.AspNetCore.Hosting;
    using Microsoft.Extensions.DependencyInjection;
    using Microsoft.AspNetCore.Builder;
    using Microsoft.AspNetCore.Routing;
    using Microsoft.Extensions.Hosting;
    using Microsoft.Extensions.Configuration;
    using Microsoft.AspNetCore.Mvc;
    using Microsoft.Extensions.Logging;

    namespace Controllers
    {
        public class Yuuna
        {
            public class Input
            {
                public string Text { get; set; }
            }

            public class Output
            {
                public Output(bool success, string message, Mood mood, string text)
                {
                    this.Success = success;
                    this.Message = message;
                    this.Mood = mood.Name;
                    this.Text = text;
                }
                public bool Success { get; }
                public string Message { get; }
                public string Mood { get; }
                public string Text { get; }
            }
        }

        [ApiController]
        [Route("[controller]")]
        public class YuunaController : ControllerBase
        {
            private readonly ILogger<YuunaController> _logger;

            public YuunaController(ILogger<YuunaController> logger, Actor actor)
            {
                _logger = logger;
                this._actor = actor;

            }

            private readonly Actor _actor;

            [HttpGet, HttpPost]
            public Yuuna.Output Post([FromBody]Yuuna.Input text)
            {
                return new Yuuna.Output(this._actor.Accept(text.Text, out var r), r.Message, r.Mood, text.Text); 
            }
        }
    }
    public class Startup
    {
        public Startup()
        {
        }


        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddSingleton(new Actor(new JiebaTextSegmenter(), new Response[]
           {
                (Moods.Sad, "我不清楚你想做什麼 OvQ"),
                (Moods.Sad, "我不懂你想幹嘛 QAQ"),
                (Moods.Sad, "我不知道你想幹嘛 OHQ"),
           }));
            services.AddControllers();
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            //app.UseHttpsRedirection();
             
            app.UseRouting();


            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });
        }
    }
    internal class EntryPoint
    {
        


        public static IHostBuilder CreateHostBuilder(string[] args) =>
               Host.CreateDefaultBuilder(args)
                   .ConfigureWebHostDefaults(webBuilder =>
                   {
                       webBuilder.UseStartup<Startup>();
                   });
        public static void Main(string[] args)
        {
            CreateHostBuilder(args).Build().Run ();

            //var actor = new Actor(new JiebaTextSegmenter(), new Response[]
            //{
            //    (Moods.Sad, "我不清楚你想做什麼 OvQ"),
            //    (Moods.Sad, "我不懂你想幹嘛 QAQ"),
            //    (Moods.Sad, "我不知道你想幹嘛 OHQ"),
            //});

            //while (true) 
            //{
            //    Console.Write("Me:  "); 
            //    if (actor.Accept(Console.ReadLine(), out var response))
            //    {
            //        Console.WriteLine("Bot: " + response.Message);
            //    }
            //    else
            //    {
            //        Console.CursorTop --; 
            //    }
            //} 
             
             
            //var canResponses = new Response[]
            //{
            //    (Moods.Sad, "我不清楚你想做什麼 OvQ"),
            //    (Moods.Sad, "我不懂你想幹嘛 QAQ"),
            //    (Moods.Sad, "我不知道你想幹嘛 OHQ"),
            //};

            //var antonyms = new[]
            //{
            //    new Antonym("是", "不是"),
            //    new Antonym("對", "不對"),
            //    new Antonym("是", "否"), 
            //};

            //IInteractiveView interactor = new ConsoleBotInteract( );
            //ITextSegmenter segmenter = new JiebaTextSegmenter();
            //IStrategy plan = new DefaultStrategy();

            //var allPlugins = ModuleManager.Instance.Modules;
            //foreach (var item in allPlugins)
            //{
            //    item.Initialize(segmenter, new GroupManager());
            //    Debug.WriteLine("已載入模組: " + item.Type.AssemblyQualifiedName);
            //}

            //// prepare task completed.
            //LOOP:
            //var text = interactor.Accept();
            //var cutted = segmenter.Cut(text);
            //Debug.WriteLine($"來自分詞器 {segmenter.Name} 的分詞結果: [ {string.Join(", ", cutted)} ]");

            //var alternative = plan.FindBest(allPlugins.Select(x => x.PatternSet).ToArray(), cutted);
            //Debug.WriteLine(alternative.Status);
            //switch (alternative.Status)
            //{
            //    // 罐頭回應
            //    case AlternativeStatus.Invalid:
            //        interactor.OnReceive(canResponses.RandomTakeOne());
            //        break;

            //    case AlternativeStatus.Optimal:
            //        {
            //            var single = alternative.Matches[0];
            //            single.Pattern.Owner.Patterns.TryGet(single.Pattern, out var bag);
            //            var r = bag.Invoke(single);
            //            interactor.OnReceive(r);
            //            break;
            //        }

            //    case AlternativeStatus.Condition:
            //        {
            //            var sb = new StringBuilder();
                        
            //            sb.Append("你是想要"); 

            //            foreach (var g in alternative.Matches[0].Pattern.ToImmutable())
            //            {
            //                sb.Append(g.RandomTakeOne().RandomTakeOne());
            //            }

            //            sb.Append("嗎?");
            //            interactor.OnReceive(sb.ToString());

            //            text = interactor.Accept();

            //            foreach (var antonym in antonyms)
            //            {
            //                if (antonym.Judge(text, out var type))
            //                {
            //                    if (type.Equals(Antonym.TypeKinds.Positive))
            //                    {
            //                        var single = alternative.Matches[0];
            //                        single.Pattern.Owner.Patterns.TryGet(single.Pattern, out var bag);
            //                        var r = bag.Invoke(single);
            //                        interactor.OnReceive(r);
            //                        goto Leave;
            //                    }
            //                    else
            //                        break;
            //                }
            //            }

            //            interactor.OnReceive(canResponses.RandomTakeOne());
            //            Leave:
            //            break;
            //        }

            //    case AlternativeStatus.Proposition:
            //        {

            //            var sb = new StringBuilder(); 
            //            var current = alternative.Matches[0];

            //            sb.Append("你是想要");

            //            foreach (var g in current.Pattern.ToImmutable())
            //            {
            //                sb.Append(g.RandomTakeOne().RandomTakeOne());
            //            }

            //            sb.Append("嗎?");
            //            interactor.OnReceive(sb.ToString());

            //            text = interactor.Accept(); 
            //            foreach (var antonym in antonyms)
            //            {
            //                if (antonym.Judge(text, out var type))
            //                {
            //                    if (type.Equals(Antonym.TypeKinds.Positive))
            //                    {
            //                        current.Pattern.Owner.Patterns.TryGet(current.Pattern, out var bag);
            //                        var r = bag.Invoke(current);
            //                        interactor.OnReceive(r);
            //                        goto Leave;
            //                    }
            //                    else if(type.Equals(Antonym.TypeKinds.Negative))
            //                        break;
            //                }
            //            }

            //            sb.Clear();


            //            current = alternative.Matches[1];
            //            sb.Append("還是");
            //            sb.Append("你是想"); 

            //            foreach (var g in current.Pattern.ToImmutable())
            //            {
            //                sb.Append(g.RandomTakeOne().RandomTakeOne());
            //            }

            //            sb.Append("?");
            //            interactor.OnReceive(sb.ToString());

            //            text = interactor.Accept();

            //            foreach (var antonym in antonyms)
            //            {
            //                if (antonym.Judge(text, out var type))
            //                {
            //                    if (type.Equals(Antonym.TypeKinds.Positive))
            //                    {
            //                        current.Pattern.Owner.Patterns.TryGet(current.Pattern, out var bag);
            //                        var r = bag.Invoke(current);
            //                        interactor.OnReceive(r);
            //                        goto Leave;
            //                    }
            //                    else if (type.Equals(Antonym.TypeKinds.Negative))
            //                        break;
            //                }
            //            }
                         
            //            interactor.OnReceive(canResponses.RandomTakeOne()); 

            //            Leave:
            //            break;
            //        }

            //    case AlternativeStatus.Paradox:


            //        break;

            //    //case AlternativeStatus.NoModuleInstalled:
            //    //    interactor.OnReceive((Moods.Sad, "無效的模組"));
            //    //    break;

            //    default:
            //        throw new InvalidOperationException();
            //}
            //goto LOOP;

            //try
            //{Send("開燈");
            //    var stt = SpeechRecognizer.Create("secret.json");
            //    stt.RecognizeCompleted += a =>
            //    {
            //        if (a.Count > 0)
            //        {
            //            Console.WriteLine("## Google 語音: " + a[0].Transcript);
            //            Send(a[0].Transcript);
            //        }
            //    };

            //    Console.WriteLine("## Proof of Concept Ver.");
            //    Console.WriteLine("可以叫我開關燈或者開關門，其他的我還不會");

            //    Console.WriteLine("按下 Enter 鍵開始錄音，任意鍵離開");
            //    Loop:
            //    if(Console.ReadKey().Key == ConsoleKey.Enter)
            //    { 
            //        using (stt.Recognize())
            //        {
            //            Console.WriteLine("按下 Enter 停止錄音，任意鍵離開"); 
            //            if(Console.ReadKey().Key == ConsoleKey.Enter)
            //                goto Loop;
            //        }
            //    } 
            //}
            //catch (Exception e)
            //{
            //    Console.WriteLine(e.Message);
            //    Console.ReadKey();
            //} 
        }

        public static void Send(string text)
        {
            

            //Console.WriteLine("我: " + text);
            //var cutted = segmenter.Cut(text);
            //Console.WriteLine($"來自分詞器 {segmenter.Name} 的分詞結果: [ {string.Join(", ", cutted)} ]");

            //var alternative = plan.FindBest(allPlugins.Select(x=>x.PatternSet).ToArray(), cutted);
            //switch (alternative.Status)
            //{
            //    // 罐頭回應
            //    case Status.Invalid:
            //        view.OnResponsed(s_canResponses.RandomTakeOne());
            //        break;

            //    case Status.Optimal:
            //        {
            //            var single = alternative.Matches[0];
            //            single.Pattern.Owner.Patterns.TryGet(single.Pattern, out var bag);
            //            var r = bag.Invoke(single);
            //            view.OnResponsed(r);
            //        }
            //        break;

            //    case Status.Condition:
            //        {
            //            Console.WriteLine("proposition");
            //            Console.Write("你是想要");
            //            foreach (var matched in alternative.Matches[0].SequentialMatches)
            //            {
            //                Console.Write(matched.ToImmutable().RandomTakeOne());
            //            }
            //            foreach (var missing in alternative.Matches[0].Missing)
            //            {
            //                foreach (var unmatch in missing.ToImmutable())
            //                {
            //                    Console.Write(unmatch.ToImmutable().RandomTakeOne());
            //                }
            //            }
            //            Console.WriteLine("嗎?");
            //            Console.ReadLine()
            //        }
            //        break;

            //    case Status.Proposition:
            //        Console.WriteLine("proposition");
            //        break;

            //    case Status.Paradox:
            //        Console.WriteLine("paradox");
            //        break;

            //    case Status.NoModuleInstalled:
            //        Console.WriteLine("noModuleInstalled");
            //        Console.WriteLine("無效的模組");
            //        break;

            //    default:
            //        throw new InvalidOperationException();
            //}



            //var list = new List<IScore>();
            //foreach (var p in allPlugins)
            //    list.AddRange(p.Evaluate(cutted));
            //list.Sort(ScoreComparer.Default);

            //var sb = new StringBuilder("Bot: ");
            //var best = list.LastOrDefault();
            //if (best is null)
            //    sb.Append("我不懂你的意思");
            //else if (best.IsCompacted)
            //{
            //    var resp = best.Owner.SelectBest(best).Invoke(best);
            //    Console.WriteLine(resp.Mood.ToString());
            //    sb.Append(resp.Message);
            //}
            //else
            //{
            //    var choices = list.Where(s => s.Missing.Count <= 1).ToImmutableArray();
            //    switch (choices.Length)
            //    {
            //        case 1:
            //            {
            //                Console.WriteLine("#1");
            //                sb.Append("你" + new[] { "是想", "想要" }.RandomTakeOne() + " ");
            //                sb.Append(choices.Select(x => x.Pattern.ToImmutable().Aggregate(string.Empty, (s, g) => s += g.ToImmutable().RandomTakeOne().ToImmutable().RandomTakeOne())));
            //                sb.Append(" 嗎?");
            //            }
            //            break;

            //        case 2:
            //            {
            //                Console.WriteLine("#2");
            //                sb.Append("你" + new[] { "是想", "想要" }.RandomTakeOne() + " ");
            //                sb.AppendJoin(" 還是 ", choices.Select(x => x.Pattern.ToImmutable().Aggregate(string.Empty, (s, g) => s += g.ToImmutable().RandomTakeOne().ToImmutable().RandomTakeOne())));
            //                sb.Append(" ?");
            //            }
            //            break;

            //        default:
            //            Console.WriteLine("#n");
            //            sb.Append("我不太清楚你想做什麼");
            //            break;
            //    }
            //}

            //Console.WriteLine(sb);
        }
    }
}